(*
 * SNU 4190.310 Programming Languages (Fall 2007)
 *
 * K-- Interpreter I
 *)

open K

signature KMINUS_PP =
  sig
    val pp: exp -> unit
  end

structure Kminus_PP : KMINUS_PP =
  struct
    fun q x = ["\"" ^ x ^ "\""]
    val pfx = "  "
    fun indent l = List.map (fn s => pfx ^ s) l
    fun comma [] = []
      | comma [h] = [h ^ ","]
      | comma (h :: t) = h :: (comma t)
    fun ps s [] = [s]
      | ps s (h :: t) =
        (s ^ "(")
          :: (List.fold_left (fn l x => (comma l) @ (indent x)) (indent h) t)
          @ [(")")]
    fun pe e =
        case e of
          NUM i => ps "NUM" [[string_of_int i]]
        | TRUE => ps "TRUE" []
        | FALSE => ps "FALSE" []
        | UNIT => ps "UNIT" []
        | VAR x => ps "VAR" [q x]
        | ADD (e1, e2) => ps "ADD" [pe e1, pe e2]
        | SUB (e1, e2) => ps "SUB" [pe e1, pe e2]
        | MUL (e1, e2) => ps "MUL" [pe e1, pe e2]
        | DIV (e1, e2) => ps "DIV" [pe e1, pe e2]
        | EQUAL (e1, e2) => ps "EQUAL" [pe e1, pe e2]
        | LESS (e1, e2) => ps "LESS" [pe e1, pe e2]
        | NOT e => ps "NOT" [pe e]
        | ASSIGN (i, e) => ps "ASSIGN" [q i, pe e]
        | SEQ (e1, e2) => ps "SEQ" [pe e1, pe e2]
        | IF (e1, e2, e3) => ps "IF" [pe e1, pe e2, pe e3]
        | WHILE (e1, e2) => ps "WHILE" [pe e1, pe e2]
        | FOR (i, e1, e2, e3) => ps "FOR" [q i, pe e1, pe e2, pe e3]
        | LETV (i, e1, e2) => ps "LETV" [q i, pe e1, pe e2]
        | LETF(f, x, e1, e2) => ps "LETF" [q f, q x, pe e1, pe e2]
        | CALLV (f, e) => ps "CALLV" [q f, pe e]
        | CALLR (f, y) => ps "CALLR" [q f, q y]
        | READ i => ps "READ" [q i]
        | WRITE e => ps "WRITE" [pe e]
        | RECORD fields => ps "RECORD" (List.map (fn (x, e) => ps "" [q x, pe e]) fields)  
        | FIELD (e, x) => ps "FIELD" [pe e, q x]
        | ASSIGNFIELD (e1, x, e2) =>  ps "ASSIGNFIELD" [pe e1, q x, pe e2]
        | MALLOC e => ps "MALLOC" [pe e]
        | AMPER x => ps "AMPER" [q x]
        | AMPERFIELD (e,x) => ps "AMPERFIELD" [pe e, q x]
        | STAR e => ps "STAR" [pe e]
        | ASSIGNSTAR (e1,e2) => ps "ASSIGNSTAR" [pe e1, pe e2]
 
fun pp e =  List.iter print_endline (pe e)
  end
